import type EventEmitter from "eventemitter3";
import type { PluginHook } from "graphile-config";
import type { ASTNode, ExecutionArgs, GraphQLArgument, GraphQLArgumentConfig, GraphQLField, GraphQLFieldConfig, GraphQLInputField, GraphQLInputFieldConfig, GraphQLInputObjectType, GraphQLInputType, GraphQLList, GraphQLNonNull, GraphQLOutputType, GraphQLScalarType, GraphQLSchema, GraphQLType } from "graphql";
import type { Bucket, RequestContext } from "./bucket.js";
import type { InputStep } from "./input.js";
import type { ExecutableStep, ListCapableStep, ModifierStep } from "./step.js";
import type { GraphileInputObjectType, GraphileObjectType } from "./utils.js";
type PromiseOrValue<T> = T | Promise<T>;
export interface GrafastOptions {
    context?: Record<string, any> | (<TContext extends Record<string, any>>(ctx: Partial<Grafast.RequestContext>, currentContext?: Partial<TContext>) => PromiseOrValue<Partial<TContext>>);
    explain?: boolean | string[];
}
declare global {
    namespace Grafast {
        interface RequestContext {
        }
        interface Context {
        }
    }
    namespace GraphileConfig {
        interface Preset {
            grafast?: GrafastOptions;
        }
        interface GrafastHooks {
            args: PluginHook<(event: {
                args: ExecutionArgs;
                ctx: Grafast.RequestContext;
                resolvedPreset: GraphileConfig.ResolvedPreset;
            }) => PromiseOrValue<ExecutionArgs>>;
        }
        interface Plugin {
            grafast?: {
                hooks?: GrafastHooks;
            };
        }
    }
}
export interface GrafastFieldExtensions {
    plan?: FieldPlanResolver<any, any, any>;
    subscribePlan?: FieldPlanResolver<any, any, any>;
}
export interface GrafastArgumentExtensions {
    inputPlan?: ArgumentInputPlanResolver;
    applyPlan?: ArgumentApplyPlanResolver;
}
export interface GrafastInputObjectTypeExtensions {
    inputPlan?: InputObjectTypeInputPlanResolver;
}
export interface GrafastInputFieldExtensions {
    inputPlan?: InputObjectFieldInputPlanResolver;
    applyPlan?: InputObjectFieldApplyPlanResolver;
}
export interface GrafastObjectTypeExtensions {
    Step?: {
        new (...args: any[]): ExecutableStep;
        displayName?: string;
    };
}
export interface GrafastEnumTypeExtensions {
}
export interface GrafastEnumValueExtensions {
    applyPlan?: EnumValueApplyPlanResolver<any>;
}
export interface GrafastScalarTypeExtensions {
    plan?: ScalarPlanResolver;
    inputPlan?: ScalarInputPlanResolver;
    idempotent?: boolean;
}
declare module "graphql" {
    interface GraphQLFieldExtensions<_TSource, _TContext, _TArgs = any> {
        graphile?: GrafastFieldExtensions;
    }
    interface GraphQLArgumentExtensions {
        graphile?: GrafastArgumentExtensions;
    }
    interface GraphQLInputObjectTypeExtensions {
        graphile?: GrafastInputObjectTypeExtensions;
    }
    interface GraphQLInputFieldExtensions {
        graphile?: GrafastInputFieldExtensions;
    }
    interface GraphQLObjectTypeExtensions<_TSource = any, _TContext = any> {
        graphile?: GrafastObjectTypeExtensions;
    }
    interface GraphQLEnumTypeExtensions {
        graphile?: GrafastEnumTypeExtensions;
    }
    interface GraphQLEnumValueExtensions {
        graphile?: GrafastEnumValueExtensions;
    }
    interface GraphQLScalarTypeExtensions {
        graphile?: GrafastScalarTypeExtensions;
    }
}
export declare const $$grafastContext: unique symbol;
export declare const $$planResults: unique symbol;
export declare const $$id: unique symbol;
export declare const $$verbatim: unique symbol;
export declare const $$bypassGraphQL: unique symbol;
export declare const $$data: unique symbol;
export declare const $$extensions: unique symbol;
export declare const $$concreteType: unique symbol;
export declare const $$idempotent: unique symbol;
export declare const $$eventEmitter: unique symbol;
export declare const $$streamMore: unique symbol;
export declare const $$proxy: unique symbol;
export declare const $$safeError: unique symbol;
export interface PolymorphicData<TType extends string = string, TData = any> {
    [$$concreteType]: TType;
    [$$data]?: TData;
}
export interface IndexByListItemStepId {
    [listItemStepId: number]: number;
}
export type GrafastValuesList<T> = ReadonlyArray<T>;
export type PromiseOrDirect<T> = PromiseLike<T> | T;
export type GrafastResultsList<T> = ReadonlyArray<PromiseOrDirect<T>>;
export type GrafastResultStreamList<T> = ReadonlyArray<PromiseOrDirect<AsyncIterable<PromiseOrDirect<T>>> | PromiseLike<never>>;
export type BaseGraphQLRootValue = any;
export interface BaseGraphQLVariables {
    [key: string]: unknown;
}
export interface BaseGraphQLArguments {
    [key: string]: any;
}
export type BaseGraphQLInputObject = BaseGraphQLArguments;
export type TargetStepOrCallback = ExecutableStep | ModifierStep | ((indexOrFieldName: number | string) => TargetStepOrCallback);
export interface FieldArgs {
    get(path?: string | string[]): ExecutableStep;
    getRaw(path?: string | string[]): InputStep;
    apply($target: TargetStepOrCallback, path?: string | string[]): void;
}
export interface FieldInfo {
    field: GraphQLField<any, any, any>;
    schema: GraphQLSchema;
}
export type FieldPlanResolver<_TArgs extends BaseGraphQLArguments, TParentStep extends ExecutableStep<any> | null, TResultStep extends ExecutableStep<any>> = ($parentPlan: TParentStep, args: FieldArgs, info: FieldInfo) => TResultStep | null;
export type InputObjectFieldInputPlanResolver<TResultStep extends ExecutableStep<any> = ExecutableStep<any>> = (input: FieldArgs, info: {
    schema: GraphQLSchema;
    entity: GraphQLInputField;
}) => TResultStep;
export type InputObjectFieldApplyPlanResolver<TFieldStep extends ExecutableStep<any> | ModifierStep<any> = ExecutableStep<any> | ModifierStep<any>, TResultStep extends ModifierStep<ExecutableStep<any> | ModifierStep<any>> | null | void = ModifierStep<ExecutableStep<any> | ModifierStep<any>> | null | void> = ($fieldPlan: TFieldStep, input: FieldArgs, info: {
    schema: GraphQLSchema;
    entity: GraphQLInputField;
}) => TResultStep;
export type InputObjectTypeInputPlanResolver = (input: FieldArgs, info: {
    schema: GraphQLSchema;
    type: GraphQLInputObjectType;
}) => ExecutableStep;
export type ArgumentInputPlanResolver<TParentStep extends ExecutableStep<any> = ExecutableStep<any>, TResultStep extends ExecutableStep<any> = ExecutableStep<any>> = ($parentPlan: TParentStep, input: FieldArgs, info: {
    schema: GraphQLSchema;
    entity: GraphQLArgument;
}) => TResultStep;
export type ArgumentApplyPlanResolver<TParentStep extends ExecutableStep<any> = ExecutableStep<any>, TFieldStep extends ExecutableStep<any> | ModifierStep<any> = ExecutableStep<any> | ModifierStep<any>, TResultStep extends ExecutableStep | ModifierStep<ExecutableStep | ModifierStep> | null | void = ExecutableStep | ModifierStep<ExecutableStep | ModifierStep> | null | void> = ($parentPlan: TParentStep, $fieldPlan: TFieldStep, input: FieldArgs, info: {
    schema: GraphQLSchema;
    entity: GraphQLArgument;
}) => TResultStep;
export type ScalarPlanResolver<TParentStep extends ExecutableStep<any> = ExecutableStep<any>, TResultStep extends ExecutableStep<any> = ExecutableStep<any>> = ($parentPlan: TParentStep, info: {
    schema: GraphQLSchema;
}) => TResultStep;
export type ScalarInputPlanResolver<TResultStep extends ExecutableStep<any> = ExecutableStep<any>> = ($inputValue: InputStep, info: {
    schema: GraphQLSchema;
    type: GraphQLScalarType;
}) => TResultStep;
export type EnumValueApplyPlanResolver<TParentStep extends ExecutableStep | ModifierStep = ExecutableStep | ModifierStep> = ($parent: TParentStep) => ModifierStep | void;
type OutputPlanForNamedType<TType extends GraphQLType> = TType extends GraphileObjectType<any, infer TStep, any> ? TStep : ExecutableStep<any>;
export type OutputPlanForType<TType extends GraphQLOutputType> = TType extends GraphQLNonNull<GraphQLList<GraphQLNonNull<infer U>>> ? ListCapableStep<any, OutputPlanForNamedType<U>> : TType extends GraphQLNonNull<GraphQLList<infer U>> ? ListCapableStep<any, OutputPlanForNamedType<U>> : TType extends GraphQLList<GraphQLNonNull<infer U>> ? ListCapableStep<any, OutputPlanForNamedType<U>> : TType extends GraphQLList<infer U> ? ListCapableStep<any, OutputPlanForNamedType<U>> : TType extends GraphQLNonNull<infer U> ? OutputPlanForNamedType<U> : OutputPlanForNamedType<TType>;
type InputPlanForNamedType<TType extends GraphQLType> = TType extends GraphileInputObjectType<any, infer U, any> ? U : ModifierStep<any>;
type InputPlanForType<TType extends GraphQLInputType> = TType extends GraphQLNonNull<GraphQLList<GraphQLNonNull<infer U>>> ? InputPlanForNamedType<U> : TType extends GraphQLNonNull<GraphQLList<infer U>> ? InputPlanForNamedType<U> : TType extends GraphQLList<GraphQLNonNull<infer U>> ? InputPlanForNamedType<U> : TType extends GraphQLList<infer U> ? InputPlanForNamedType<U> : TType extends GraphQLNonNull<infer U> ? InputPlanForNamedType<U> : InputPlanForNamedType<TType>;
type InputTypeForNamedType<TType extends GraphQLType> = TType extends GraphQLScalarType<infer U> ? U : any;
type InputTypeFor<TType extends GraphQLInputType> = TType extends GraphQLNonNull<GraphQLList<GraphQLNonNull<infer U>>> ? InputTypeForNamedType<U> : TType extends GraphQLNonNull<GraphQLList<infer U>> ? InputTypeForNamedType<U> : TType extends GraphQLList<GraphQLNonNull<infer U>> ? InputTypeForNamedType<U> : TType extends GraphQLList<infer U> ? InputTypeForNamedType<U> : TType extends GraphQLNonNull<infer U> ? InputTypeForNamedType<U> : InputTypeForNamedType<TType>;
export type GraphileFieldConfig<TType extends GraphQLOutputType, TContext extends Grafast.Context, TParentStep extends ExecutableStep<any> | null, TFieldStep extends ExecutableStep<any>, TArgs extends BaseGraphQLArguments> = Omit<GraphQLFieldConfig<any, any>, "args" | "type"> & {
    type: TType;
    plan?: FieldPlanResolver<TArgs, TParentStep, TFieldStep>;
    subscribePlan?: FieldPlanResolver<TArgs, TParentStep, TFieldStep>;
    args?: GraphileFieldConfigArgumentMap<TType, TContext, TParentStep, TFieldStep>;
};
export type GraphileFieldConfigArgumentMap<_TType extends GraphQLOutputType, TContext extends Grafast.Context, TParentStep extends ExecutableStep<any> | null, TFieldStep extends ExecutableStep<any>> = {
    [argName: string]: GraphileArgumentConfig<any, TContext, TParentStep, TFieldStep, any, any>;
};
export type GraphileArgumentConfig<TInputType extends GraphQLInputType, _TContext extends Grafast.Context, _TParentStep extends ExecutableStep<any> | null, TFieldStep extends ExecutableStep<any>, _TArgumentStep extends TFieldStep extends ExecutableStep<any> ? ModifierStep<TFieldStep> | null : null, _TInput extends InputTypeFor<TInputType>> = Omit<GraphQLArgumentConfig, "type"> & {
    type: TInputType;
    inputPlan?: ArgumentInputPlanResolver<any>;
    applyPlan?: ArgumentApplyPlanResolver<any, any>;
};
export type GraphileInputFieldConfig<TInputType extends GraphQLInputType, _TContext extends Grafast.Context, _TParentStep extends ModifierStep<any>, _TResultStep extends InputPlanForType<TInputType>, _TInput extends InputTypeFor<TInputType>> = Omit<GraphQLInputFieldConfig, "type"> & {
    type: TInputType;
    inputPlan?: InputObjectFieldInputPlanResolver;
    applyPlan?: InputObjectFieldApplyPlanResolver<any>;
};
export type TrackedArguments<TArgs extends BaseGraphQLArguments = BaseGraphQLArguments> = {
    get<TKey extends keyof TArgs>(key: TKey): InputStep;
};
export interface StepStreamOptions {
    initialCount: number;
}
export interface StepOptions {
    stream: StepStreamOptions | null;
}
export interface StepOptimizeOptions {
    stream: StepStreamOptions | null;
}
export type GrafastSubscriber<TTopics extends {
    [key: string]: any;
} = {
    [key: string]: any;
}> = {
    subscribe<TTopic extends keyof TTopics = keyof TTopics>(topic: TTopic): PromiseOrDirect<AsyncIterableIterator<TTopics[TTopic]>>;
};
export interface NodeIdCodec<T = any> {
    encode(value: T): string | null;
    decode(value: string): T;
}
export type NodeIdHandler<TCodecs extends {
    [key: string]: NodeIdCodec<any>;
} = {
    [key: string]: NodeIdCodec<any>;
}, TCodecName extends keyof TCodecs = keyof TCodecs, TNodeStep extends ExecutableStep<any> = ExecutableStep<any>, TSpec = any> = {
    typeName: string;
    codecName: TCodecName & string;
    match(specifier: TCodecs[TCodecName] extends NodeIdCodec<infer U> ? U : any): boolean;
    plan($thing: TNodeStep): ExecutableStep<TCodecs[TCodecName] extends NodeIdCodec<infer U> ? U : any>;
    getSpec(plan: ExecutableStep<TCodecs[TCodecName] extends NodeIdCodec<infer U> ? U : any>): TSpec;
    get(spec: TSpec): TNodeStep;
    deprecationReason?: string;
};
export type BaseEventMap = Record<string, any>;
export type EventMapKey<TEventMap extends BaseEventMap> = string & keyof TEventMap;
export type EventCallback<TPayload> = (params: TPayload) => void;
export interface TypedEventEmitter<TEventMap extends BaseEventMap> extends EventEmitter<any, any> {
    addListener<TEventName extends EventMapKey<TEventMap>>(eventName: TEventName, callback: EventCallback<TEventMap[TEventName]>): this;
    on<TEventName extends EventMapKey<TEventMap>>(eventName: TEventName, callback: EventCallback<TEventMap[TEventName]>): this;
    once<TEventName extends EventMapKey<TEventMap>>(eventName: TEventName, callback: EventCallback<TEventMap[TEventName]>): this;
    removeListener<TEventName extends EventMapKey<TEventMap>>(eventName: TEventName, callback: EventCallback<TEventMap[TEventName]>): this;
    off<TEventName extends EventMapKey<TEventMap>>(eventName: TEventName, callback: EventCallback<TEventMap[TEventName]>): this;
    emit<TEventName extends EventMapKey<TEventMap>>(eventName: TEventName, params: TEventMap[TEventName]): boolean;
}
export type ExecutionEventMap = {
    explainOperation: {
        operation: Record<string, any> & {
            type: string;
            title: string;
        };
    };
};
export type ExecutionEventEmitter = TypedEventEmitter<ExecutionEventMap>;
export interface ExecutionExtra {
    meta: Record<string, unknown>;
    eventEmitter: ExecutionEventEmitter | undefined;
    _bucket: Bucket;
    _requestContext: RequestContext;
}
export interface LocationDetails {
    node: ASTNode | readonly ASTNode[];
    parentTypeName: string | null;
    fieldName: string | null;
}
export type JSONValue = boolean | number | string | null | JSONObject | JSONArray;
export interface JSONObject {
    [key: string]: JSONValue;
}
export interface JSONArray extends Array<JSONValue> {
}
export {};
//# sourceMappingURL=interfaces.d.ts.map